/* eslint-disable fp/no-delete -- immer */
/* eslint-disable fp/no-let */
import { nanoid } from "nanoid";
import { createContext } from "./react-immersive-history";
import { buildPage, } from "./interfaces";
import { showConditionToConditionGroup } from "./utils/filtrexToFormula";
import { v4 as uuidv4 } from "uuid";
import { omit } from "./utils/omit";
export const Home = buildPage();
Home.type = "navigation";
export const transformQorebaseSchema = (schema) => {
    var _a;
    // delete large unused data from column
    const transformColumn = (column_) => {
        const column = omit(["metadata", "isHidden", "description"], column_);
        const columnDefinition = column.definition &&
            omit([
                "default",
                "nullable",
                "unique",
                "textType",
                "triggers",
                "exact",
                "function",
                "expression",
            ], column_.definition);
        switch (column.type) {
            case "action":
                return Object.assign(Object.assign({}, column), { definition: (columnDefinition === null || columnDefinition === void 0 ? void 0 : columnDefinition.action) === "JavaScript"
                        ? omit(["function"], columnDefinition)
                        : (columnDefinition === null || columnDefinition === void 0 ? void 0 : columnDefinition.action) === "Print"
                            ? omit(["template"], columnDefinition)
                            : (columnDefinition === null || columnDefinition === void 0 ? void 0 : columnDefinition.action) === "Operations"
                                ? omit(["operations"], columnDefinition)
                                : columnDefinition });
            default:
                return Object.assign(Object.assign({}, column), { definition: columnDefinition });
        }
    };
    return {
        tables: schema.tables.map((table) => ({
            name: table.name,
            columns: table.columns.map(transformColumn),
        })),
        views: schema.tables.flatMap((table) => (table.views || []).map((view) => {
            var _a;
            return ({
                name: view.name,
                // @ts-ignore
                query: view.query,
                columns: (((_a = view.query) === null || _a === void 0 ? void 0 : _a.fields) || [])
                    .map((columnName) => table.columns.find((column) => column.name === columnName))
                    .filter(Boolean)
                    .map(transformColumn),
            });
        })),
        insights: schema.tables.flatMap((table) => (table.insights || []).map((insight) => ({
            name: insight.name,
            columns: insight.columns,
            table: table.name,
            sorts: insight.sorts,
            insight: insight.insight,
            params: insight.params,
            registeredParams: insight.registeredParams,
        }))),
        // @ts-ignore
        queries: (_a = schema.queries) !== null && _a !== void 0 ? _a : [],
    };
};
export const normalizeApp = (app) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
    const MAX_BOTTOM_NAV = 5;
    const pages = {};
    const convertedNav = {
        bottom: [],
        sidetop: { root: [], items: {} },
    };
    for (const page of Object.values(app.maps.page)) {
        if (page.id) {
            pages[page.id] = Object.assign(Object.assign(Object.assign({}, buildPage()), page), { 
                // @ts-ignore it was marked as "special" before
                type: page.type === "special" ? "system" : page.type, defaultRow: page.defaultRow.type === "row-id"
                    ? Object.assign(Object.assign({}, page.defaultRow), { rowId: page.defaultRow.rowId ||
                            (typeof ((_a = page.params) === null || _a === void 0 ? void 0 : _a.rowId) === "string"
                                ? (_b = page.params) === null || _b === void 0 ? void 0 : _b.rowId
                                : "") }) : page.defaultRow, params: page.params || {
                    rowId: ((_c = page.defaultRow) === null || _c === void 0 ? void 0 : _c.type) === "row-id" ? page.defaultRow.rowId : "",
                }, navbar: Object.assign(Object.assign({}, buildPage().navbar), page.navbar) });
            // convert old nav to new nav context
            if (["navigation", "menu"].includes(page.type)) {
                const { id, title, icon } = page;
                const sidetopId = nanoid(3);
                convertedNav.sidetop.root.push(sidetopId);
                convertedNav.sidetop.items[sidetopId] = {
                    id: sidetopId,
                    children: [],
                    hasChildren: false,
                    isExpanded: false,
                    isChildrenLoading: false,
                    data: {
                        label: title,
                        icon: icon,
                        page: id,
                        linked: true,
                    },
                };
                if (convertedNav.bottom.length < MAX_BOTTOM_NAV) {
                    convertedNav.bottom.push({
                        icon: icon,
                        label: title,
                        page: id,
                    });
                }
                pages[page.id].type = undefined;
            }
        }
    }
    const components = Object.fromEntries(Object.entries(app.maps.component).map(([id, component]) => [
        id,
        Object.assign(Object.assign({}, component), { showCondition: Array.isArray(component.showCondition)
                ? showConditionToConditionGroup(component.showCondition)
                : component.showCondition }),
    ]));
    return Object.assign(Object.assign({}, app), { maps: Object.assign(Object.assign({}, app.maps), { page: pages, component: components }), integrations: app.integrations || {}, customSources: app.customSources || {}, theme: Object.assign(Object.assign({}, initialApp.theme), app.theme), navbar: {
            bottom: Object.assign(Object.assign(Object.assign({}, initialApp.navbar.bottom), (_d = app.navbar) === null || _d === void 0 ? void 0 : _d.bottom), { content: [
                    ...((_f = (_e = app.navbar) === null || _e === void 0 ? void 0 : _e.bottom.content) !== null && _f !== void 0 ? _f : []),
                    ...convertedNav.bottom,
                ].slice(0, MAX_BOTTOM_NAV) }),
            sidetop: Object.assign(Object.assign(Object.assign({}, initialApp.navbar.sidetop), (_g = app.navbar) === null || _g === void 0 ? void 0 : _g.sidetop), { content: Object.assign(Object.assign(Object.assign({}, initialApp.navbar.sidetop.content), (_h = app.navbar) === null || _h === void 0 ? void 0 : _h.sidetop.content), { rootId: "root", items: Object.assign(Object.assign(Object.assign(Object.assign({}, initialApp.navbar.sidetop.content.items), (_j = app.navbar) === null || _j === void 0 ? void 0 : _j.sidetop.content.items), convertedNav.sidetop.items), { root: Object.assign(Object.assign({}, initialApp.navbar.sidetop.content.items.root), { children: [
                                ...((_l = (_k = app.navbar) === null || _k === void 0 ? void 0 : _k.sidetop.content.items.root.children) !== null && _l !== void 0 ? _l : []),
                                ...convertedNav.sidetop.root,
                            ] }) }) }) }),
        }, schema: Object.assign(Object.assign({}, initialApp.schema), app.schema), components: app.components || initialApp.components });
};
export const initialSchema = {
    tables: [],
    views: [],
    insights: [],
    queries: [],
};
export const initialApp = {
    name: "Application",
    description: "",
    icon: "",
    theme_color: "#049fdc",
    config: {
        projectID: "unknown",
        appID: "",
        endpoint: "",
        id: uuidv4(),
        initialPage: {
            entryPage: "",
            loggedInPage: "",
        },
    },
    integrations: {},
    maps: {
        component: {
            login_component: {
                id: "login_component",
                componentID: "Login",
                options: {},
                properties: {},
                showCondition: { type: "all", conditions: [] },
                disableCondition: { type: "all", conditions: [] },
                source: {
                    type: "list",
                    params: {},
                    sorts: {},
                },
                metadata: {},
            },
            register_text_input: {
                componentID: "Text Input",
                options: {},
                showCondition: { type: "all", conditions: [] },
                disableCondition: { type: "all", conditions: [] },
                id: "register_text_input",
                properties: {
                    label: "Email",
                    placeholder: "Enter your email here",
                    hidden: false,
                    defaultValue: "",
                    data: {
                        inputID: "external_id",
                        field: {
                            type: "string",
                            options: {
                                format: "text",
                            },
                        },
                        validation: {
                            required: false,
                        },
                        type: "field",
                    },
                },
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
                metadata: {},
            },
            register_password_input: {
                componentID: "Password",
                options: {},
                showCondition: { type: "all", conditions: [] },
                disableCondition: { type: "all", conditions: [] },
                id: "register_password_input",
                properties: {
                    title: "Password",
                    placeholder: "Enter your password here",
                    confirmPassword: true,
                    confirmTitle: "Confirm Password",
                    confirmPlaceholder: "Reenter your password here",
                    data: {
                        inputID: "password",
                        field: {
                            type: "string",
                            options: {
                                format: "text",
                            },
                        },
                        validation: {
                            required: true,
                        },
                        type: "field",
                    },
                },
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
                metadata: {},
            },
            register_role_input: {
                componentID: "Role",
                options: {},
                showCondition: { type: "all", conditions: [] },
                disableCondition: { type: "all", conditions: [] },
                id: "register_role_input",
                properties: {
                    roleId: "",
                },
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
                metadata: {},
            },
            forbidden_component: {
                id: "forbidden_component",
                componentID: "Forbidden",
                options: {},
                showCondition: { type: "all", conditions: [] },
                disableCondition: { type: "all", conditions: [] },
                properties: {},
                metadata: {},
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
            },
        },
        page: {
            login: Object.assign(Object.assign({}, buildPage()), { id: "login", components: [{ component: "login_component" }], type: "system", title: "Login" }),
            register: Object.assign(Object.assign({}, buildPage()), { id: "register", components: [
                    {
                        component: "register_text_input",
                    },
                    {
                        component: "register_password_input",
                    },
                    {
                        component: "register_role_input",
                    },
                ], type: "system", title: "Register", sourceID: "users", mode: "form" }),
            forbidden: Object.assign(Object.assign({}, buildPage()), { id: "forbidden", components: [{ component: "forbidden_component" }], type: "system", title: "Forbidden" }),
        },
    },
    theme: {
        id: "Light",
        options: { primaryColor: "#1890ff" },
        navigationLayout: "sidebar",
        fonts: {
            heading: ["Inter", ""],
            body: ["Inter", ""],
        },
    },
    navbar: {
        bottom: {
            showLabel: true,
            content: [],
        },
        sidetop: {
            position: "sidebar",
            collapsible: false,
            content: {
                rootId: "root",
                items: {
                    root: {
                        id: "root",
                        children: [],
                        hasChildren: true,
                        isExpanded: true,
                        isChildrenLoading: false,
                        data: {
                            label: "root tree",
                            icon: "",
                            page: "",
                            linked: true,
                        },
                    },
                },
            },
        },
    },
    schema: initialSchema,
    customSources: {},
    components: [],
};
export const RuntimeContext = createContext(initialApp, (modify) => {
    try {
        return {
            setTheme: (theme) => {
                modify((draft) => {
                    draft.theme = theme;
                });
            },
            setApp: (app) => {
                modify(() => {
                    return normalizeApp(app);
                });
            },
            setIcon: (url) => {
                modify((draft) => {
                    draft.icon = url;
                }, false);
            },
            setInitialPage: (entryPage) => {
                modify((draft) => {
                    draft.config.initialPage = Object.assign(Object.assign({}, draft.config.initialPage), { entryPage });
                }, false);
            },
            setLoggedInPage: (loggedInPage) => {
                modify((draft) => {
                    draft.config.initialPage = Object.assign(Object.assign({}, draft.config.initialPage), { loggedInPage });
                }, false);
            },
            addNpmComponent: (name, version) => {
                modify((draft) => {
                    if (draft.components.find((c) => c.name === name && c.version === version)) {
                        return;
                    }
                    draft.components.push({
                        name,
                        version,
                        enabled: true,
                    });
                });
            },
            removeNpmComponent: (name) => {
                modify((draft) => {
                    const index = draft.components.findIndex((c) => c.name === name);
                    if (index !== -1) {
                        draft.components.splice(index, 1);
                    }
                });
            },
            setVersionNpmComponent: (name, version) => {
                modify((draft) => {
                    const index = draft.components.findIndex((c) => c.name === name);
                    if (index !== -1) {
                        draft.components[index].version = version;
                    }
                });
            },
            setEnableNpmComponent: (name, enabled) => {
                modify((draft) => {
                    const index = draft.components.findIndex((c) => c.name === name);
                    if (index !== -1) {
                        draft.components[index].enabled = enabled;
                    }
                });
            },
            addPage(page) {
                modify((draft) => {
                    draft.maps.page[page.id] = page;
                });
            },
            addIntegration(integration) {
                modify((draft) => {
                    draft.integrations[integration.name] = integration;
                });
            },
            setIntegrationProperty: function (namespace, field, value) {
                modify((draft) => {
                    draft.integrations[namespace].properties.values[field] = value;
                });
            },
            removeIntegration(namespace) {
                modify((draft) => {
                    delete draft.integrations[namespace];
                });
            },
            addCustomSources(externalData) {
                modify((draft) => {
                    draft.customSources[externalData.name] = externalData;
                });
            },
            removeCustomSources(namespace) {
                modify((draft) => {
                    delete draft.customSources[namespace];
                });
            },
            addComponent(component, pageRef, pageOpt) {
                modify((draft) => {
                    draft.maps.component[component.id] = component;
                    if (pageRef && pageOpt && typeof pageOpt.idx === "number") {
                        draft.maps.page[pageRef.page].components.splice(pageOpt.idx, 0, {
                            component: component.id,
                        });
                    }
                    else if (pageRef) {
                        draft.maps.page[pageRef.page].components.push({
                            component: component.id,
                        });
                    }
                });
            },
            addPageAndComponents(page, components) {
                modify((draft) => {
                    draft.maps.page[page.id] = page;
                    components.forEach((component) => {
                        draft.maps.component[component.id] = component;
                    });
                });
            },
            swapComponent(target, sourceIndex, destinationIndex, deletionLength = 1) {
                modify((draft) => {
                    const components = draft.maps.page[target.page].components;
                    const removed = components.splice(sourceIndex, deletionLength);
                    removed && components.splice(destinationIndex, 0, ...removed);
                    draft.maps.page[target.page].components = components;
                });
            },
            moveComponents(listUpdateComponents) {
                modify((draft) => {
                    listUpdateComponents.forEach((comp) => {
                        draft.maps.component[comp.id].metadata = comp.metadata;
                    });
                });
            },
            swapPage(targetPages, sourceIndex, destinationIndex) {
                modify((draft) => {
                    const pages = Object.values(draft.maps.page).filter((page) => page.type !== targetPages[0].type);
                    const [removed] = targetPages.splice(sourceIndex, 1);
                    removed && targetPages.splice(destinationIndex, 0, removed);
                    const newPages = pages.concat(targetPages);
                    draft.maps.page = newPages.reduce((reduced, page) => (Object.assign(Object.assign({}, reduced), { [page.id]: page })), {});
                });
            },
            setComponentProperty: function (target, field, value) {
                modify((draft) => {
                    draft.maps.component[target.component].properties[field] = value;
                });
            },
            setComponentMetadata: function (target, field, value) {
                modify((draft) => {
                    if (draft.maps.component[target.component].metadata) {
                        draft.maps.component[target.component].metadata[field] = value;
                    }
                    else {
                        draft.maps.component[target.component].metadata = {
                            [field]: value,
                        };
                    }
                });
            },
            setPageProperty: function (target, field, value) {
                modify((draft) => {
                    draft.maps.page[target.page].properties.values[field] = value;
                });
            },
            removeComponent(target, from, option) {
                modify((draft) => {
                    var _a;
                    if ((_a = option === null || option === void 0 ? void 0 : option.layoutDetail) === null || _a === void 0 ? void 0 : _a.componentsID) {
                        option.layoutDetail.componentsID.forEach((compId) => {
                            if (draft.maps.component[compId].metadata) {
                                draft.maps.component[compId].metadata.layout = undefined;
                            }
                            else {
                                draft.maps.component[compId].metadata = {};
                            }
                        });
                    }
                    const children = Object.entries(draft.maps.component).filter(([, value]) => value.metadata.container === target.component);
                    const childrenKeys = children.map((item) => item[0]);
                    draft.maps.page[from.page].components = draft.maps.page[from.page].components.filter((ref) => ![...childrenKeys, target.component].includes(ref.component));
                    childrenKeys.forEach((key) => {
                        delete draft.maps.component[key];
                    });
                    delete draft.maps.component[target.component];
                });
            },
            removeComponents(target, from) {
                modify((draft) => {
                    draft.maps.page[from.page].components = draft.maps.page[from.page].components.filter((ref) => !target.find((comp) => comp.component === ref.component));
                    target.forEach(({ component }) => {
                        delete draft.maps.component[component];
                    });
                });
            },
            removePage(target) {
                modify((draft) => {
                    const page = draft.maps.page[target.page];
                    for (const component of page.components) {
                        delete draft.maps.component[component.component];
                    }
                    delete draft.maps.page[target.page];
                });
            },
            setComponentSource(target, source) {
                modify((draft) => {
                    draft.maps.component[target.component].source = source;
                });
            },
            setComponentLayoutMetadata(target, column) {
                modify((draft) => {
                    if (!draft.maps.component[target.component].metadata ||
                        !draft.maps.component[target.component].metadata.layout)
                        return;
                    draft.maps.component[target.component].metadata.layout.column =
                        column;
                });
            },
            removeComponentLayoutMetadata(target) {
                modify((draft) => {
                    if (!draft.maps.component[target.component].metadata ||
                        !draft.maps.component[target.component].metadata.layout)
                        return;
                    draft.maps.component[target.component].metadata.layout = undefined;
                });
            },
            addComponentShowCondition(target) {
                modify((draft) => {
                    if (draft.maps.component[target.component]) {
                        draft.maps.component[target.component].showCondition.conditions.push({
                            id: nanoid(),
                            formula: "true == true",
                        });
                    }
                });
            },
            addComponentDisableCondition(target) {
                modify((draft) => {
                    if (!draft.maps.component[target.component].disableCondition) {
                        draft.maps.component[target.component].disableCondition = {
                            type: "all",
                            conditions: [],
                        };
                    }
                    if (draft.maps.component[target.component]) {
                        draft.maps.component[target.component].disableCondition.conditions.push({
                            id: nanoid(),
                            formula: "true == true",
                        });
                    }
                });
            },
            updatePage(target, update) {
                modify((draft) => {
                    update(draft.maps.page[target.page]);
                });
            },
            updateComponentShowCondition(target, update) {
                modify((draft) => {
                    if (draft.maps.component[target.component]) {
                        update(draft.maps.component[target.component].showCondition);
                    }
                });
            },
            updateComponentDisableCondition(target, update) {
                modify((draft) => {
                    if (draft.maps.component[target.component]) {
                        update(draft.maps.component[target.component].disableCondition);
                    }
                });
            },
            removeComponentShowCondition(target, index) {
                modify((draft) => {
                    if (draft.maps.component[target.component]) {
                        draft.maps.component[target.component].showCondition.conditions =
                            draft.maps.component[target.component].showCondition.conditions.filter((cond, i) => i !== index);
                    }
                });
            },
            removeComponentDisableCondition(target, index) {
                modify((draft) => {
                    if (draft.maps.component[target.component]) {
                        draft.maps.component[target.component].disableCondition.conditions = draft.maps.component[target.component].disableCondition.conditions.filter((cond, i) => i !== index);
                    }
                });
            },
            setPageTitle(target, title) {
                modify((draft) => {
                    draft.maps.page[target.page].title = title;
                });
            },
            setShowNavbar(target, type, toggle) {
                modify((draft) => {
                    draft.maps.page[target.page].navbar[type].show = toggle;
                });
            },
            setSideTopNavbar(value) {
                modify((draft) => {
                    draft.navbar.sidetop.position = value;
                });
            },
            setCollapsible(value) {
                modify((draft) => {
                    draft.navbar.sidetop.collapsible = value;
                });
            },
            setBottomNavLabel(toggle) {
                modify((draft) => {
                    draft.navbar.bottom.showLabel = toggle;
                });
            },
            addBottomNav(nav) {
                modify((draft) => {
                    draft.navbar.bottom.content.push(nav);
                });
            },
            removeBottomNav(index) {
                modify((draft) => {
                    draft.navbar.bottom.content.splice(index, 1);
                });
            },
            setBottomNavContent(idx, key, value) {
                modify((draft) => {
                    draft.navbar.bottom.content[idx][key] = value;
                });
            },
            swapBottomNavContent(source, destination) {
                modify((draft) => {
                    const newNav = [...draft.navbar.bottom.content];
                    const targetNav = newNav[source];
                    newNav.splice(source, 1);
                    newNav.splice(destination, 0, targetNav);
                    draft.navbar.bottom.content = newNav;
                });
            },
            swapSidetopNavContent(newTree) {
                modify((draft) => {
                    draft.navbar.sidetop.content = newTree;
                });
            },
            addSidetopNav(newNav) {
                modify((draft) => {
                    draft.navbar.sidetop.content.items.root.children.push(newNav.id);
                    draft.navbar.sidetop.content.items[newNav.id] = newNav;
                });
            },
            setSidetopNavbarContent(navId, key, value) {
                modify((draft) => {
                    draft.navbar.sidetop.content.items[navId].data[key] = value;
                });
            },
            removeSidetopNav(nav) {
                modify((draft) => {
                    const sideTop = Object.assign({}, draft.navbar.sidetop.content.items);
                    if (nav.hasChildren) {
                        sideTop[nav.id].children.forEach((id) => {
                            if (sideTop[id].hasChildren) {
                                sideTop[id].children.forEach((subId) => delete sideTop[subId]);
                            }
                            delete sideTop[id];
                        });
                    }
                    delete sideTop[nav.id];
                    for (const key in sideTop) {
                        if (sideTop[key].children.find((e) => e === nav.id)) {
                            const idx = sideTop[key].children.findIndex((e) => e === nav.id);
                            sideTop[key].children.splice(idx, 1);
                            if (sideTop[key].children.length === 0) {
                                sideTop[key].hasChildren = false;
                                sideTop[key].isExpanded = false;
                            }
                        }
                    }
                    draft.navbar.sidetop.content.items = sideTop;
                });
            },
        };
    }
    catch (error) { }
}, { enableHistory: true, trackChanges: true, trackPaths: ["app"] });
//# sourceMappingURL=RuntimeContext.js.map